// Tile engine
// THIS ENGINE ONLY REQUIRES ALLEGRO
#include <allegro.h>
#include "math.h"
#include "iostream"

void playsample(int i);


// Bitmap pointers
BITMAP *buffer;
BITMAP *background;
BITMAP *backgroundmasked;
BITMAP *title;
BITMAP *puppyimage;

const int puppywidth=150;
const int puppyheight=125;

const int screenwidth=640;
const int screenheight=480;
const float newlevelcount=40;

float newleveltimer=0;

const float maxprogress=20;
enum PuppyState {ps_ok,ps_worried,ps_jumpingoff};
class Puppy {
public:
	int x;
	int y;
	int health;
	int tx;
	int ty;
	float progress;//max 15, say
	int width;
	int height;
	PuppyState state;
	int jumpamtx;
	int jumpamty;
	int halfyiffed;
	bool master;
	bool hidehoff;
	Puppy(int px, int py,int ptx, int pty, int prog, bool pmaster):x(px),y(py),tx(ptx),ty(pty),progress((prog>maxprogress)?maxprogress:prog),width(puppywidth), height(puppyheight),state(ps_ok),halfyiffed(0),master(pmaster),hidehoff(pmaster)
	{
		//		jumpamtx=(rand()%11)-5;
		//		if (jumpamtx==0 && jumpamty==0)
		{
			jumpamtx=3;
			jumpamty=3;
		}
	};
	bool draw();
};
//return true to delete

Puppy* puppyarray[20];
int puppycount=0;

bool Puppy::draw()
{
	
	
	
	if (master)
	{
		
		if (puppycount==1)
			master=false;
		
		if (hidehoff)
		{
			const float r=( 1-(newlevelcount-5)/newlevelcount);
			masked_stretch_blit(backgroundmasked, buffer, 0, 0, screenwidth, screenheight,(1-r)*410 , (1-r)*331, r*screenwidth, r*screenheight);
		}
		
		masked_blit(puppyimage,buffer,0,0,x,y,width,height);
		
	}
	else
	{
		if (halfyiffed<50)
			halfyiffed++;
		
		if (state==ps_ok)
		{
			//		rect(buffer,x,y,x+width,y+height,makecol((progress/maxprogress)*255,0,0));
			masked_blit(puppyimage,buffer,0,0,x,y,width,height);
			if (progress<maxprogress)
				progress+=0.1;
			
		}
		else if (state==ps_worried)
		{
			
			if (progress>0)
				progress--;
			else 
			{
				state=ps_jumpingoff;
				playsample(2);
			}
			
			if (halfyiffed==50)
			{
				halfyiffed=0;
				playsample(rand()%2);
			}
			
			//		rect(buffer,x,y,x+width,y+height,makecol(0,(progress/maxprogress)*255,0));
			masked_blit(puppyimage,buffer,0,0,x,y,width,height);
			
		}
		else if (state==ps_jumpingoff)
		{
			
			if (hidehoff)
			{
				const float r=( 1-(newlevelcount-5)/newlevelcount);
				masked_stretch_blit(backgroundmasked, buffer, 0, 0, screenwidth, screenheight,(1-r)*410 , (1-r)*331, r*screenwidth, r*screenheight);
			}
			x+=jumpamtx;
			y+=jumpamty;
			if (x>screenwidth || y>screenheight || (x+width)<0 || (y+height)<0)
				return true;
			masked_blit(puppyimage,buffer,0,0,x,y,width,height);
			
			//		rect(buffer,x,y,x+width,y+height,makecol(0,0,255));
			
		}
	}
	return false;
}


int oldmousex;
int oldmousey;
int velocitytimer;
// variables
int endgame;
int p1x;
int p1y;
int p1h;
int p2x;
int p2y;
int p2h;
int p1charge;
int p2charge;
bool p1won;
bool p2won;
bool tick;
char flashticker;
int level;
int pausetimer;
bool gameover;
bool waiting;

bool canpress=true;

char history[57][47][32];
int p1healthhistory[57];
int p2healthhistory[57];
char blast[47][32];
char newblast[47][32];

int curtime;

char mode;  //0 title, 1 game;
bool p1dead;
bool p2dead;



int max(int a, int b)
{
	if (a<=b) return b;
	else return a;
}

int min(int a, int b)
{
	if (a>=b) return b;
	else return a;
}

int d(int x1, int y1, int x2, int y2)
{
	return (int) (sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)));
} 

/* FRAMERATE CONTROL*/


volatile unsigned int frames, fps;
void second_counter(void)
{
	fps=frames;
	frames=0;
} END_OF_FUNCTION(second_counter);
int timewaster_delay;
extern void do_prog(void);

volatile unsigned int ticks;
void ticker(void)
{
	++ticks;
} END_OF_FUNCTION(ticker);


//---------------------------------------------------------
// INITIALIZATION FUNCTION
//---------------------------------------------------------

void reset_variables()
{
	
	oldmousex=mouse_x;
	oldmousey=mouse_y;
	velocitytimer=0;
	puppycount=0;
	
	puppyarray[puppycount]=new Puppy(359,271,359,271,100,true);
	puppycount++;
	
	puppyarray[puppycount]=new Puppy(301,335,301,335,100,false);
	puppycount++;
	
	puppyarray[puppycount]=new Puppy(210,108,210,108,100,false);
	puppycount++;	
	
	
	endgame=0;
	p1x = 1;
	p1y = 1;
	p1h = 100;
	p2x = 30;
	p2y = 45;
	p2h = 100;
	tick=true;
	flashticker = 0;
	level = 1;
	pausetimer=50;
	gameover=false;
	p1won=false;
	p2won=false;
	p1dead=false;
	p2dead=false;	
	mode=0;
	curtime=0;
	waiting=true;
}


int panning = 128;
int pitch = 1000;
int volume = 128;
SAMPLE *sample[10];

void playsample(int i)
{
    play_sample(sample[i], volume, panning, pitch, FALSE);	
}

void init ()
{
	// Install keyboard & timer
	install_keyboard();
	install_timer();
	
	//install a digital sound driver
    if (install_sound(DIGI_AUTODETECT, MIDI_NONE, "") != 0) 
    {
        allegro_message("Error initializing sound system");
        return;
    }
    sample[0] = load_sample("wav/yiff.wav");
    sample[1] = load_sample("wav/yiff2.wav");
    sample[2] = load_sample("wav/yiff3.wav");
	//    sample[3] = load_sample("wav/music.wav");
	
	//    play_sample(sample[3], volume, panning, pitch, TRUE);	
	
	reset_variables();
}


//---------------------------------------------------------
// DRAW BUFFER
//---------------------------------------------------------

// showBuffer copies the contents of the buffer bitmap to the actual screen
void showBuffer ()
{
	// Blit to screen
	blit (buffer, screen, 0, 0, 0, 0, 1024, 768);
}

//---------------------------------------------------------
// INPUT
//---------------------------------------------------------

void overlap(int x, int y, Puppy *puppy)
{
	if ((puppy->state==ps_ok)&&
		(x>=puppy->x)&&
		(y>=puppy->y)&&
		(x<puppy->x + puppy->width)&&
		(y<puppy->y + puppy->height))
		puppy->state=ps_worried;
	else
	{
		if (puppy->state==ps_worried) 
			puppy->state=ps_ok;
	}
	
}

void input ()
{
	
	if (key[KEY_SPACE]&&mode==0)
		mode=2;
	
	velocitytimer=(velocitytimer+1) %10;
	poll_mouse();
	
	if (velocitytimer==0)
	{
		oldmousex=mouse_x;
		oldmousey=mouse_y;
	}
	
	
	{
		
		//collision stuff
		
		if ((mouse_x!=oldmousex)||(mouse_y!=oldmousey))
		{
			for (int i=0;i<puppycount;i++)
			{
				overlap(mouse_x,mouse_y,puppyarray[i]);
			}
		}
		else
			for (int i=0;i<puppycount;i++)
			{
				if (puppyarray[i]->state==ps_worried)
					puppyarray[i]->state=ps_ok;
			}
		
		//int velocity=(oldmousex-mouse_x)*(oldmousex-mouse_x)+(oldmousey-mouse_y)*(oldmousey-mouse_y);
		
	}
	
	// If esc is pressed, exit
	if (key[KEY_ESC])
		endgame=1;
}


void draw_title()
{
	blit(title,buffer,0,0,0,0,screenwidth,screenheight);
}

void removepuppy(int i)
{
	
	
	delete puppyarray[i];
	
	for (int j=i+1;j<puppycount;j++)
		puppyarray[j-1]=puppyarray[j];
	
    puppycount--;
	if (puppycount==0)
		newleveltimer=newlevelcount-5;	
	
};

void draw_map(int temp)
{
	blit(background,buffer,0,0,0,0,screenwidth,screenheight);
	for (int i=0;i<puppycount;i++)
		if (puppyarray[i]->draw())
		{
			removepuppy(i);
			i--;
		}
	rectfill(buffer,mouse_x,mouse_y,mouse_x+3,mouse_y+3,makecol(255,255,255));
}


//---------------------------------------------------------
// INITIALIZE GRAPHICS
//---------------------------------------------------------

// initialise our graphics routnine
int initGraphics ()
{
	set_color_depth (32);
	if (set_gfx_mode(GFX_AUTODETECT_FULLSCREEN,screenwidth, screenheight, 0, 0))
	{
		// generate an error message
		// note: allegro_message can be used on any platform
		allegro_message ("Error: Could not initialize graphics mode!");
		return 0;
		
	}
	
	return 1;
}



void health_check()
{
	if (blast[p1y][p1x]>0)
	{
		blast[p1y][p1x]=0;
		p1h=max(p1h-10,0);
	}
	
	if (blast[p2y][p2x]>0)
	{
		blast[p2y][p2x]=0;
		p2h=max(p2h-10,0);
	}
	
	if (p1h==0) 
		p1dead=true;
	if (p2h==0)
		p2dead=true;
}
//------------------------------------------------------------
//------------------------------------------------------------
// MAIN
//------------------------------------------------------------
//------------------------------------------------------------
int temp;
int main (void)
{
	// initialize allegro
	allegro_init ();
	
	// call initialization function
	init();
	
	
	
	LOCK_FUNCTION(second_counter);
	LOCK_VARIABLE(fps);
	LOCK_VARIABLE(frames);
	LOCK_FUNCTION(ticker);
	LOCK_VARIABLE(ticks);
	
	frames=timewaster_delay=0;
	
	install_int_ex(second_counter, BPS_TO_TIMER(1));
	install_int_ex(ticker, BPS_TO_TIMER(40));
	install_mouse() ;
	if (initGraphics())
	{
		// create our buffer
		buffer=create_bitmap (1024, 768);
		background=load_bitmap("bmp/level1main.bmp",NULL);
		backgroundmasked=load_bitmap("bmp/hoffmask.bmp",NULL);
		title=load_bitmap("bmp/title.bmp",NULL);
		puppyimage=load_bitmap("bmp/puppy2.bmp",NULL);
		
		while (endgame==0)
		{
			curtime++;
			curtime%=57;
			
			// check for input
			if (newleveltimer>0)
			{
				newleveltimer--;
				
				blit(background,buffer,0,0,0,0,screenwidth,screenheight);
				
				float r = 1-newleveltimer/newlevelcount;
				masked_stretch_blit(backgroundmasked, buffer, 0, 0, screenwidth, screenheight,(1-r)*410 , (1-r)*331, r*screenwidth, r*screenheight);
				if (newleveltimer<=0)
				{
					reset_variables();
					mode=2;
				}
			}
			else
			{
				input();
				
				if (mode==0)
				{
					draw_title();
				}
				else if (mode==2)
				{
					
					health_check();
					
					// draw tile map
					temp++;
					draw_map(temp);
					
				}
			}
			
			
			/* control timing */
			while (ticks<1)
			{
				rest(0);
			}
			ticks=0;
			++frames;
			
			// Show the buffer & clear the buffer
			showBuffer ();
			
			// clear buffer
			clear(buffer);                               
		}
		
	}
	// clean up allegro
	allegro_exit ();
	return 0;
}
END_OF_MAIN ();


